/*
 *
 * Five toggle channels depending on pulse count.
 *
 * Author Jean-Pierre PARISY <jpparisy@free.fr>
 *
 * Copyright (C) 2012
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <htc.h>
#include <pic.h>
#include <pic12f509.h>

#define TEMPO 20

// set fuse bits
 __CONFIG(MCLRE_OFF & CP_OFF & WDT_OFF & OSC_IntRC);

unsigned char cnt, tempo, sequence = 0, prev = 2;
bit sGP0 = 0, sGP1 = 0, sGP2 = 0, sGP4 = 0;

void decode_sequence (unsigned char last_value);

/* ************************************************************************* */

main (void)
{
	OPTION = ~T0CS & ~PSA & (nGPWU | nGPPU | T0SE | 0x2);
//	T0CS  = internal cycle clock (4 Mhz), this allows use of GP4 as port
//	PSA   = prescaler assigned to timer 
//	nGPWU = wake up disabled
//	nGPPU = pull up disabled
//	T0SE  = high to low source edge
//	prescaler values:	+-----+-------+
//						| 0x0 | 1:2   |
//						| 0x1 | 1:4   |
//						| 0x2 | 1:8   | <---
//						| 0x3 | 1:16  |
//						| 0x4 | 1:32  |
//						| 0x5 | 1:64  |
//						| 0x6 | 1:128 |
//						| 0x7 | 1:256 |
//						+-----+-------+	
//	Timer = 256 usec without prescaler
//	256 us * 8 = 2048 usec
//	Signal = 1.5 ms +/- 0.5 ms  => 1000 usec :: 2000 usec

	TRIS = 0b101000;						// GP3 and GP5 as input, all other ports as output
	GPIO = 0;								// all ports off
	for (cnt = 0; cnt < 5; cnt++) {			// wait for five pulses
		do {} while (!GP3);					// wait start of pulse
		do {} while (GP3);					// wait end of pulse
	}
/* ************************************************************************* */
	for (;;) {								// loop for ever
		do {} while (!GP3);					// wait start of pulse
		TMR0 = 255 - 100;					// wait at least 100 * 8 = 800 usec
		do {} while (TMR0 >= 2);			// time elapsed, TMR0 has reached 0 
		do {} while (GP3);					// wait end of pulse
		cnt = TMR0;							// TMR0 is now between 18 and 156, neutral is 88
/* ************************************************************************* */
		decode_sequence (cnt < 88);			// count and decode number of switch toggles
		GP0 = sGP0;							// set port 0
		GP1 = sGP1;							// set port 1
		GP2 = sGP2;							// set port 2
		GP4 = sGP4;							// set port 4
	}
}

/* ************************************************************************* */

void decode_sequence (unsigned char last_value)
{
	if (prev != 2) {						// initial value
		if (prev ^= last_value) {			// each time the value is changed,
			sequence++; 					// the sequence number is incremented
			tempo = TEMPO;					// initialize temporisation
		}
		else if (!tempo-- && sequence) {	// when the value doesn't change,  wait before accepting the command
			if (!GP5) {						// GP5 state
				switch (sequence) {			// set ports according to the sequence number
				case 1: sGP0=~sGP0; break;	// toggle port 0
				case 2: sGP1=~sGP1; break;	// toggle port 1
				case 3: sGP2=~sGP2; break;	// toggle port 2
				case 4: sGP4=~sGP4; break;	// toggle port 4
				}
			}
			else {
				switch (sequence) {			// set ports according to the sequence number
				case 5: sGP0=~sGP0; break;	// toggle port 0
				case 6: sGP1=~sGP1; break;	// toggle port 1
				case 7: sGP2=~sGP2; break;	// toggle port 2
				case 8: sGP4=~sGP4; break;	// toggle port 4
				}
			}
			sequence = 0;
		}
	}
	prev = last_value;
}
/* ************************************************************************* */
